IF EXISTS (SELECT *
                   FROM SYSOBJECTS O, SYSUSERS U
                 WHERE O.NAME = 'DIST_AUTO_PRAT' AND
                             TYPE = 'P' AND
                             U.UID = O.UID AND
                             U.NAME = USER
                )
DROP PROCEDURE DIST_AUTO_PRAT
GO

CREATE PROCEDURE DIST_AUTO_PRAT
(
   @P_NRDISTCARD NUMERIC,   
   @P_NRSEQPRATCAR NUMERIC,
   @P_TIPO_DIST_AUTO VARCHAR(10)
)
AS
   DECLARE @V_CHAVE VARCHAR(100);
   /*Dados do prato no cardpio*/   
   DECLARE @V_CDFILIAL VARCHAR(4);
   DECLARE @V_CDSERVFILI VARCHAR(5);
   DECLARE @V_CDCLIENTE VARCHAR(21);
   DECLARE @V_CDDESTFILI VARCHAR(3);
   DECLARE @V_CDFILAUXPRAT VARCHAR(4);   
   DECLARE @V_CDPRATO VARCHAR(10);
   DECLARE @V_VRPECONSCARD FLOAT;
   DECLARE @V_QTPORPESSOA FLOAT; 
   DECLARE @V_QTEXTRA FLOAT;
   
   /*============================*/
   /*=====  VARIVEIS DE CONTROLE  =====*/
   /*============================*/   
   DECLARE @V_CONF_A_UTILIZAR INT;
   
   DECLARE @V_INDICE INT;
   
   /*Chave da embalagem na distribuio do cardpio*/
   DECLARE @V_NREMBDISTCARD NUMERIC;
   DECLARE @V_NREMBDISTCARD_J_EXISTENTE NUMERIC;
   
   /*Sequencial da novo cdigo*/   
   DECLARE @V_SEQUENCIAL VARCHAR(20);
   
   /*Valores totais a distribuir*/
   DECLARE @V_TOTAL_COMENSAIS FLOAT;
   DECLARE @V_TOTAL_A_DISTRIBUIR FLOAT;
   
   /*Quantidade total e quantidade de embalagens completas*/
   DECLARE @V_QTDE_EMB FLOAT;
   DECLARE @V_QTDE_EMB_COMP FLOAT;
   DECLARE @V_RESTO FLOAT;
   
   /*Comensais correspondentes*/
   DECLARE @V_COMENSAIS FLOAT;
   DECLARE @V_C FLOAT;
   
   /*Amonstra na embalagem*/
   DECLARE @V_EXTRA_EMB FLOAT;
   DECLARE @V_A FLOAT;
   DECLARE @V_RESTO_A FLOAT;   
   DECLARE @V_QTEMB_A_COMP FLOAT;
   
   /*Quantidade do prato alocada na embalagem*/
   DECLARE @V_QTDE_PRAT FLOAT;
   DECLARE @V_QP FLOAT;      
   DECLARE @V_RESTO_PRAT FLOAT;   
   DECLARE @V_QTEMB_PRAT_COMP FLOAT;
   
   /*Fator utilizado como peso para determinao da prioridade da embalagem*/   
   /*Corresponde a 2 - (quantidade ocupada da embalagem incompleta) */
   DECLARE @V_FATOR_QTDE_N_APROVEITADA FLOAT;
   DECLARE @V_ORDEM_PREFERENCIA FLOAT;
   
   /*Embalagem com o menor valor de ordem (preferida)*/
   DECLARE @V_CDCONTAINER_ESCOLHIDA VARCHAR(5);
   DECLARE @V_VRQTDEPRATEMB_ESCOLHIDA FLOAT;
   DECLARE @V_ORDEM_PREFERENCIA_ESCOLHIDA FLOAT;
   
   /*Espao no aproveitado da embalagem incompleta*/
   DECLARE @V_DISPONIVEL FLOAT;
   /*======================================*/
   /*=====  CURSOR DE EMBALAGENS A CONSIDERAR  =====*/
   /*                                                                                   */
   /* A varivel V_CONF_A_UTILIZAR controla a execuo          */
   /*======================================*/
   DECLARE @V_CDCONTAINER VARCHAR(5);
   DECLARE @V_VRMINCONTAINER FLOAT;
   DECLARE @V_NRORDEM NUMERIC;   
   DECLARE @V_VRQTDEPRATEMB FLOAT;

BEGIN   
   /*======================================*/
   /*===== BUSCA O PRATO DO  CARDAPIO============*/
   /* Garante que o prato est no cardpio informado               */
   /*======================================*/   

   SELECT @V_CDFILIAL = C.CDFILIAL, @V_CDSERVFILI = C.CDSERVFILI, @V_CDFILAUXPRAT = CC.CDFILAUXPRAT, @V_CDPRATO = CC.CDPRATO, 
          @V_TOTAL_COMENSAIS = CNF.QTTOTCOMENSAIS, @V_QTPORPESSOA = CNF.QTPORPESSOA, @V_QTEXTRA = CNF.QTTOTEXTRA, 
          @V_TOTAL_A_DISTRIBUIR = CNF.QTTOTPRATO, @V_CDCLIENTE = D.CDCLIENTE, @V_CDDESTFILI = D.CDDESTFILI
   FROM DISTCARD D
        INNER JOIN CARDADIA C ON C.CDFILIAL = D.CDFILIAL AND
                                 C.NRCARDAPIO = D.NRCARDAPIO
        LEFT JOIN CARDACLI CL ON CL.CDFILIAL = C.CDFILIAL AND
                                  CL.NRCARDAPIO = C.NRCARDAPIO AND
                                  CL.CDCLIENTE = D.CDCLIENTE AND
                                  CL.CDDESTFILI = D.CDDESTFILI
        INNER JOIN COMPCARD CC ON CC.CDFILIAL = C.CDFILIAL AND
                                  CC.NRCARDAPIO = C.NRCARDAPIO AND
                                  CC.NRSEQPRATCAR = @P_NRSEQPRATCAR                                     
        INNER JOIN CNFDISTCARD CNF ON CNF.NRDISTCARD = D.NRDISTCARD AND
                                      CNF.NRSEQPRATCAR = @P_NRSEQPRATCAR          
   WHERE D.NRDISTCARD = @P_NRDISTCARD;
   
   IF (@V_CDFILIAL IS NULL) OR (@V_QTPORPESSOA IS NULL) BEGIN
      RETURN;
   END;
   
   IF (@V_QTEXTRA IS NULL) BEGIN
      SET @V_QTEXTRA = 0;
   END;
   
   /*======================================*/
   /*=====  DETERMINA EMBALAGENS A CONSIDERAR =====*/
   /*======================================*/   
   SET @V_CONF_A_UTILIZAR = NULL;
   
   SELECT @V_CONF_A_UTILIZAR = MAX(ISNULL(CONF_A_UTILIZAR, 0))      
   FROM
   (
      /*Restries a nvel de servio*/
      SELECT 1 AS CONF_A_UTILIZAR      
      WHERE EXISTS(SELECT 1 
                     FROM RESTEMBTPPRAT
                     WHERE CDFILIAL = @V_CDFILIAL
                          AND CDSERVFILI = @V_CDSERVFILI
                          AND CDCLIENTE IS NULL)
   UNION
      /*Restries a nvel de ponto de entrega*/
      SELECT 2 AS CONF_A_UTILIZAR      
      WHERE EXISTS(SELECT 1 
                     FROM RESTEMBTPPRAT
                     WHERE CDFILIAL = @V_CDFILIAL
                          AND CDSERVFILI = @V_CDSERVFILI
                          AND CDCLIENTE = @V_CDCLIENTE
                          AND CDDESTFILI = @V_CDDESTFILI)
   UNION
      /*Restries a nvel de prato*/
      SELECT 3 AS CONF_A_UTILIZAR      
      WHERE EXISTS(SELECT 1
                     FROM RESTEMBPRAT
                     WHERE CDFILIAL = @V_CDFILIAL
                          AND CDSERVFILI = @V_CDSERVFILI
                          AND CDFILAUXPRAT = @V_CDFILAUXPRAT
                          AND CDPRATO = @V_CDPRATO)
   ) S;
   
   IF @V_CONF_A_UTILIZAR IS NULL BEGIN
      SET @V_CONF_A_UTILIZAR = 0;
   END;
   
   /*======================================*/
   /*=====  CURSOR DE EMBALAGENS A CONSIDERAR  =====*/
   /*                                                                                   */
   /* A varivel V_CONF_A_UTILIZAR controla a execuo          */
   /*======================================*/
   
   DECLARE C_EMBALAGENS CURSOR LOCAL
   FOR
      /*Relaes com embalagens existentes para o prato*/
      SELECT R.CDCONTAINER, NULL AS VRMINCONTAINER,
                  999999 AS ORDEM,
                  R.VRQTDEPRATEMB
      FROM RELPRATEMB R
      WHERE R.CDFILAUXPRAT = @V_CDFILAUXPRAT
           AND R.CDPRATO = @V_CDPRATO
           AND @V_CONF_A_UTILIZAR = 0
      
      UNION
      
      /*Restries a nvel de servio*/
      SELECT I.CDCONTAINER, I.VRMINCONTAINER,
                  CASE WHEN I.VRMINCONTAINER IS NULL THEN 999999 ELSE I.NRORDEM END AS ORDEM,
                  REL.VRQTDEPRATEMB
        FROM  RESTEMBTPPRAT R                  
                  INNER JOIN ITRESTEMBTPPRAT I ON I.NRRESTEMBTPPRAT = R.NRRESTEMBTPPRAT                  
                  INNER JOIN RELPRATEMB REL ON REL.CDFILAUXPRAT = @V_CDFILAUXPRAT AND
                                                                   REL.CDPRATO = @V_CDPRATO AND
                                                                   REL.CDCONTAINER = I.CDCONTAINER                                                               
        WHERE R.CDFILIAL = @V_CDFILIAL
             AND R.CDSERVFILI = @V_CDSERVFILI
             AND R.CDCLIENTE IS NULL
             AND @V_CONF_A_UTILIZAR = 1
      
      UNION
      
      /*Restries a nivel de ponto de entrega*/
      SELECT I.CDCONTAINER, I.VRMINCONTAINER,
                  CASE WHEN I.VRMINCONTAINER IS NULL THEN 999999 ELSE I.NRORDEM END AS ORDEM,
                  REL.VRQTDEPRATEMB
        FROM  RESTEMBTPPRAT R
                  INNER JOIN ITRESTEMBTPPRAT I  ON I.NRRESTEMBTPPRAT = R.NRRESTEMBTPPRAT                   
                  INNER JOIN RELPRATEMB REL ON REL.CDFILAUXPRAT = @V_CDFILAUXPRAT AND
                                                                   REL.CDPRATO = @V_CDPRATO AND
                                                                   REL.CDCONTAINER = I.CDCONTAINER                                                               
        WHERE R.CDFILIAL = @V_CDFILIAL
             AND R.CDSERVFILI = @V_CDSERVFILI
             AND R.CDCLIENTE = @V_CDCLIENTE
             AND R.CDDESTFILI = @V_CDDESTFILI
             AND @V_CONF_A_UTILIZAR = 2
             
      UNION
      
      /*Restries a nivel de prato*/
      SELECT I.CDCONTAINER, I.VRMINCONTAINER,
                  CASE WHEN I.VRMINCONTAINER IS NULL THEN 999999 ELSE I.NRORDEM END AS ORDEM,
                  REL.VRQTDEPRATEMB
      FROM RESTEMBPRAT R
                INNER JOIN ITRESTEMBPRAT I ON I.NRRESTEMBPRAT = R.NRRESTEMBPRAT
                INNER JOIN RELPRATEMB REL ON REL.CDFILAUXPRAT = @V_CDFILAUXPRAT AND
                                                                 REL.CDPRATO = @V_CDPRATO AND
                                                                 REL.CDCONTAINER = I.CDCONTAINER
      WHERE R.CDFILIAL = @V_CDFILIAL
           AND R.CDSERVFILI = @V_CDSERVFILI
           AND R.CDFILAUXPRAT = @V_CDFILAUXPRAT
           AND R.CDPRATO = @V_CDPRATO
           AND @V_CONF_A_UTILIZAR = 3
           
      ORDER BY ORDEM;
   
   /*======================================*/
   /*=====  CONTAINERS COM O MNIMO INFORMADO  =====*/   
   /*======================================*/   
      
   OPEN C_EMBALAGENS;
   FETCH NEXT FROM C_EMBALAGENS         
    INTO @V_CDCONTAINER, @V_VRMINCONTAINER, @V_NRORDEM, @V_VRQTDEPRATEMB;
   
   WHILE (@@FETCH_STATUS = 0) AND  ((@V_TOTAL_A_DISTRIBUIR <> 0) OR (@V_QTEXTRA <> 0)) AND (@V_VRMINCONTAINER IS NOT NULL) BEGIN
      
      SET @V_INDICE = 0;
      
      SET @V_QTDE_EMB = 0;
      SET @V_COMENSAIS = 0;
      SET @V_QTDE_PRAT = 0;
      SET @V_EXTRA_EMB = 0;
   
      /*"Preenche" cada embalagem at atingir o mnimo, reduzindo o total a distribuir a cada iterao*/
      WHILE (@V_INDICE <> @V_VRMINCONTAINER) AND ((@V_TOTAL_A_DISTRIBUIR <> 0) OR (@V_QTEXTRA <> 0)) BEGIN
         
         /*Nmero de embalagens*/
         SET @V_QTDE_EMB = @V_QTDE_EMB + 1;
         
         /*Distribuio do peso do prato*/
         IF (@V_TOTAL_A_DISTRIBUIR > 0) BEGIN
            /*Se embalagem maior que o total do prato a distribuir*/
            IF (@V_VRQTDEPRATEMB > @V_TOTAL_A_DISTRIBUIR) BEGIN
               /*Joga todo o prato na embalagem*/
               SET @V_QP = @V_TOTAL_A_DISTRIBUIR;
               SET @V_QTDE_PRAT = @V_QTDE_PRAT + @V_QP;
               SET @V_COMENSAIS = @V_COMENSAIS + @V_TOTAL_COMENSAIS;
               
                /*Abate do total de comensais e do total a distribuir o valor do prato naquela embalagem*/
               SET @V_TOTAL_A_DISTRIBUIR = 0;
               SET @V_TOTAL_COMENSAIS = 0;
            END
            /*Se embalagem menor ou igual ao valor total do prato*/            
            ELSE BEGIN
               /*Abate do total a distribuir o valor da capacidade da embalagem (em relao ao prato)*/
               SET @V_QP = @V_VRQTDEPRATEMB;
               SET @V_QTDE_PRAT = @V_QTDE_PRAT + @V_QP;
               SET @V_C = ROUND(@V_QP / @V_QTPORPESSOA, 5);
               IF @V_C > @V_TOTAL_COMENSAIS BEGIN
                  SET @V_C = @V_TOTAL_COMENSAIS;
               END;
               SET @V_COMENSAIS = @V_COMENSAIS + @V_C;
               
               /*Abate do total de comensais e do total a distribuir o valor do prato naquela embalagem*/
                SET @V_TOTAL_A_DISTRIBUIR = @V_TOTAL_A_DISTRIBUIR - @V_QP;
                SET @V_TOTAL_COMENSAIS = @V_TOTAL_COMENSAIS - @V_C;
            END;
            
            /*Verifica se h espao na embalagem incompleta para amostra*/
            IF ((@V_TOTAL_A_DISTRIBUIR = 0) AND (@V_QTEXTRA > 0)) BEGIN
               SET @V_A = @V_VRQTDEPRATEMB - @V_QP;
               
               IF (@V_A > @V_QTEXTRA) BEGIN
                  SET @V_A = @V_QTEXTRA;
               END;
               
               /*Aloca amostra na embalagem e abate do total de amostra a distribuir*/
               SET @V_EXTRA_EMB = @V_EXTRA_EMB + @V_A;
               
               SET @V_QTEXTRA = @V_QTEXTRA - @V_A;
               
            END;
         END
         /*Quando somente a amostra estiver sobrando para distribuir*/
         ELSE BEGIN
            SET @V_A = @V_VRQTDEPRATEMB;
               
            IF (@V_A > @V_QTEXTRA) BEGIN
               SET @V_A = @V_QTEXTRA;
            END;
            
            SET @V_EXTRA_EMB = @V_EXTRA_EMB + @V_A;
            
            SET @V_QTEXTRA = @V_QTEXTRA - @V_A;
         END;

         SET @V_INDICE = @V_INDICE + 1;
      END;

      /*Busca novo sequencial*/               
      SET @V_CHAVE = 'EMBDISTCARD';
      EXEC NOVO_CODIGO_SC @V_CHAVE,@V_SEQUENCIAL OUTPUT;
      SET @V_NREMBDISTCARD = CONVERT(NUMERIC(20,5), @V_SEQUENCIAL);

      /*Insere embalagem na distribuio do cardpio*/
      INSERT INTO EMBDISTCARD
      (NRDISTCARD, NREMBDISTCARD, CDCONTAINER, QTCONTAINER, IDCADAUTO)
      VALUES 
      (@P_NRDISTCARD, @V_NREMBDISTCARD, @V_CDCONTAINER, @V_QTDE_EMB, 'S');
      
      /*Insere o prato na embalagem na distribuio*/
      INSERT INTO ITEMBDISTCARD
      (NRDISTCARD, NRSEQPRATCAR, NREMBDISTCARD, 
       QTCOMENSAIS, QTPRATO, QTEXTRA, IDCADAUTO, IDPRINCIPAL)
      VALUES
      (@P_NRDISTCARD, @P_NRSEQPRATCAR, @V_NREMBDISTCARD, 
       @V_COMENSAIS, @V_QTDE_PRAT, @V_EXTRA_EMB, 'S', 'S');

      FETCH NEXT FROM C_EMBALAGENS         
      INTO @V_CDCONTAINER, @V_VRMINCONTAINER, @V_NRORDEM, @V_VRQTDEPRATEMB;
      
   END;
   
   CLOSE C_EMBALAGENS;
   
   /*===================================================*/
   /*=====  ESCOLHE O MELHOR CONTAINER PARA DISTRIBUIR O PRATO  =====*/
   /*===================================================*/
   
   WHILE (@V_TOTAL_A_DISTRIBUIR <> 0) OR (@V_QTEXTRA <> 0) BEGIN
      SET @V_CDCONTAINER_ESCOLHIDA = 'NDEF';
      SET @V_VRQTDEPRATEMB_ESCOLHIDA  = 0;
      SET @V_ORDEM_PREFERENCIA_ESCOLHIDA = 999999;

      SET @V_COMENSAIS = 0;
      SET @V_QTDE_PRAT = 0;
      SET @V_EXTRA_EMB = 0;
      
      OPEN C_EMBALAGENS;
      
      /*===================================================*/
      /*=====  PERCORRE TODAS AS EMBALAGENS DISPONVEIS =====*/
      /*=====  CALCULANDO A ORDEM DE PREFERNCIA ============*/
      /*===================================================*/
      
      FETCH NEXT FROM C_EMBALAGENS         
       INTO @V_CDCONTAINER, @V_VRMINCONTAINER, @V_NRORDEM, @V_VRQTDEPRATEMB;
       
      WHILE (@@FETCH_STATUS = 0) BEGIN      
         
         /*Quantidade de embalagens completas*/
         SET @V_QTDE_EMB_COMP = ROUND((@V_TOTAL_A_DISTRIBUIR + @V_QTEXTRA) / @V_VRQTDEPRATEMB, 0, 1);

         /*Quantidade na embalagem incompleta*/
         SET @V_RESTO = CONVERT(NUMERIC(20,5), @V_TOTAL_A_DISTRIBUIR + @V_QTEXTRA) % CONVERT(NUMERIC(20,5), @V_VRQTDEPRATEMB);
         SET @V_RESTO = ROUND(@V_RESTO, 5);

         /*Fator de no aproveitamento da embalagem incompleta*/
         SET @V_FATOR_QTDE_N_APROVEITADA = 1 + (@V_VRQTDEPRATEMB - @V_RESTO)/@V_VRQTDEPRATEMB;
         
         /*Ordem da embalagem na preferncia : quantidade de embalagens + se o resto no for zero, 1 * fator de no aproveitamento*/
         SET @V_ORDEM_PREFERENCIA = @V_QTDE_EMB_COMP + CASE @V_RESTO WHEN 0 THEN 0 ELSE @V_FATOR_QTDE_N_APROVEITADA END;
         /*Se a ordem for menor (melhor), armazena os dados nas variveis auxiliares*/
         IF @V_ORDEM_PREFERENCIA <= @V_ORDEM_PREFERENCIA_ESCOLHIDA BEGIN            
            SET @V_ORDEM_PREFERENCIA_ESCOLHIDA = @V_ORDEM_PREFERENCIA;
            SET @V_CDCONTAINER_ESCOLHIDA = @V_CDCONTAINER;
            SET @V_VRQTDEPRATEMB_ESCOLHIDA = @V_VRQTDEPRATEMB;
         END;
         
         FETCH NEXT FROM C_EMBALAGENS         
          INTO @V_CDCONTAINER, @V_VRMINCONTAINER, @V_NRORDEM, @V_VRQTDEPRATEMB;

      END;
      
      CLOSE C_EMBALAGENS;
      
      /*Se no h embalagens a ser escolhida, termina a funo*/
      IF @V_CDCONTAINER_ESCOLHIDA = 'NDEF' BEGIN
         RETURN;
      END;
      
      /*Verifica se o prato j est associado  embalagem*/
      /*Verifica se h espao disponvel para alocao de mais quantidade na embalagem*/
      SET @V_NREMBDISTCARD_J_EXISTENTE = NULL;
      SET @V_DISPONIVEL = 0;
      
      SELECT @V_NREMBDISTCARD_J_EXISTENTE = D.NREMBDISTCARD, 
             @V_DISPONIVEL = ROUND(R.VRQTDEPRATEMB - (CONVERT(NUMERIC(20,5), I.QTPRATO + I.QTEXTRA) % CONVERT(NUMERIC(20,5), R.VRQTDEPRATEMB)), 5)
        FROM EMBDISTCARD D
            INNER JOIN ITEMBDISTCARD I ON I.NREMBDISTCARD = D.NREMBDISTCARD
            INNER JOIN RELPRATEMB R ON R.CDCONTAINER = D.CDCONTAINER AND
                                       R.CDFILAUXPRAT = @V_CDFILAUXPRAT AND
                                       R.CDPRATO = @V_CDPRATO
        WHERE D.NRDISTCARD = @P_NRDISTCARD
          AND D.CDCONTAINER = @V_CDCONTAINER_ESCOLHIDA
          AND I.NRSEQPRATCAR = @P_NRSEQPRATCAR;
      
      
      
      /*Escolhida a melhor embalagem, determina a quantidade de embalagens, o quanto do prato est na embalagem */      
      SET @V_QTEMB_PRAT_COMP = ROUND(@V_TOTAL_A_DISTRIBUIR / @V_VRQTDEPRATEMB_ESCOLHIDA, 0, 1);      
      SET @V_RESTO_PRAT = ROUND(CONVERT(NUMERIC(20,5), @V_TOTAL_A_DISTRIBUIR) % CONVERT(NUMERIC(20,5), @V_VRQTDEPRATEMB_ESCOLHIDA), 5);
      
      
      /*Quantidade do prato na embalagem (parcial, falta o resto que ser tratado a seguir)*/
      SET @V_QP = @V_QTEMB_PRAT_COMP * @V_VRQTDEPRATEMB_ESCOLHIDA;
      SET @V_QTDE_PRAT = @V_QP;
      
      /*Comensais correspondente  quantidade do prato*/
      SET @V_C = ROUND(@V_QP/@V_QTPORPESSOA, 5);
      IF @V_C > @V_TOTAL_COMENSAIS BEGIN
         SET @V_C = @V_TOTAL_COMENSAIS;
      END;
      SET @V_COMENSAIS = @V_C;
      
      /*Determina o quanto de amostra a embalagem ir transportar*/
      SET @V_QTEMB_A_COMP = ROUND(@V_QTEXTRA / @V_VRQTDEPRATEMB_ESCOLHIDA, 0, 1);
      SET @V_RESTO_A = ROUND(CONVERT(NUMERIC(20,5), @V_QTEXTRA) % CONVERT(NUMERIC(20,5), @V_VRQTDEPRATEMB_ESCOLHIDA), 5);
      /*Quantidade da amostra na embalagem (parcial, falta o resto que ser tratado a seguir)*/      
      SET @V_EXTRA_EMB = @V_QTEMB_A_COMP * @V_VRQTDEPRATEMB_ESCOLHIDA;
      
      /*Quantidade de embalagens*/
      SET @V_QTDE_EMB = @V_QTEMB_PRAT_COMP + @V_QTEMB_A_COMP;
      
      /*===================================================*/
      /*=====  VERIFICAO DO RESTANTE A DISTRIBUIR ======*/      
      /*===================================================*/
      IF @V_NREMBDISTCARD_J_EXISTENTE IS NOT NULL BEGIN
         /*Se a disponibilidade  igual  capacidade, ento  uma nova embalagem*/
         IF @V_DISPONIVEL = @V_VRQTDEPRATEMB_ESCOLHIDA AND @P_TIPO_DIST_AUTO <> 'DISTR'  BEGIN
            SET @V_QTDE_EMB = @V_QTDE_EMB + 1;
         END;
         
         /*Aloca parte ou todo o resto na embalagem incompleta j existente na distribuio*/         
         IF @V_DISPONIVEL > 0 AND 
            ((@P_TIPO_DIST_AUTO <> 'DISTR') OR 
             ( (@P_TIPO_DIST_AUTO = 'DISTR') AND  @V_DISPONIVEL <> @V_VRQTDEPRATEMB_ESCOLHIDA))  BEGIN
            IF @V_RESTO_PRAT > @V_DISPONIVEL BEGIN
               SET @V_RESTO_PRAT = @V_RESTO_PRAT - @V_DISPONIVEL;               
               SET @V_QTDE_PRAT = @V_QTDE_PRAT + @V_DISPONIVEL;
               
               /*Comensais correspondentes ao resto da diviso*/
               SET @V_C = ROUND(@V_DISPONIVEL/@V_QTPORPESSOA, 5);
               IF @V_C > @V_TOTAL_COMENSAIS BEGIN
                  SET @V_C = @V_TOTAL_COMENSAIS;
               END;
               SET @V_COMENSAIS = @V_COMENSAIS + @V_C;
               
               SET @V_DISPONIVEL = 0;
            END
            ELSE BEGIN
               SET @V_DISPONIVEL = @V_DISPONIVEL - @V_RESTO_PRAT;               
               SET @V_QTDE_PRAT = @V_QTDE_PRAT + @V_RESTO_PRAT;
               
               /*Comensais correspondentes ao resto da diviso*/
               SET @V_C = ROUND(@V_RESTO_PRAT/@V_QTPORPESSOA, 5);
               IF @V_C > @V_TOTAL_COMENSAIS BEGIN
                  SET @V_C = @V_TOTAL_COMENSAIS;
               END;
               SET @V_COMENSAIS = @V_COMENSAIS + @V_C;
               
               SET @V_RESTO_PRAT = 0;
            END;
         END;
         
         /*Aloca parte ou todo o resto da amostra na embalagem incompleta j existente na distribuio*/         
         IF @V_DISPONIVEL > 0 BEGIN
            IF @V_RESTO_A > @V_DISPONIVEL BEGIN
               SET @V_RESTO_A = @V_RESTO_A - @V_DISPONIVEL;               
               SET @V_EXTRA_EMB = @V_EXTRA_EMB + @V_DISPONIVEL;
               SET @V_DISPONIVEL = 0;
            END
            ELSE BEGIN
               SET @V_DISPONIVEL = @V_DISPONIVEL - @V_RESTO_A;               
               SET @V_EXTRA_EMB = @V_EXTRA_EMB + @V_RESTO_A;
               SET @V_RESTO_A = 0;
            END;
         END;
      END;
      
      /*Verifica se a soma dos restos preenche uma embalagem*/
      IF @V_RESTO_PRAT + @V_RESTO_A >= @V_VRQTDEPRATEMB_ESCOLHIDA BEGIN
         /*INCLUI MAIS UMA EMBALAGEM COMPLETA*/
         SET @V_QTDE_EMB = @V_QTDE_EMB + 1;
         
         SET @V_QTDE_PRAT = @V_QTDE_PRAT + @V_RESTO_PRAT;
         
         /*Comensais correspondentes ao resto da diviso*/
         SET @V_C = ROUND(@V_RESTO_PRAT/@V_QTPORPESSOA, 5);
         IF @V_C > @V_TOTAL_COMENSAIS BEGIN
            SET @V_C = @V_TOTAL_COMENSAIS;
         END;
         SET @V_COMENSAIS = @V_COMENSAIS + @V_C;
         
         /*O resto da amostra completa a embalagem at onde der*/
         /*assim o que passar da soma do resto  abatido do resto da amostra a distribuir*/
         SET @V_RESTO_A = @V_RESTO_A - (@V_VRQTDEPRATEMB_ESCOLHIDA - @V_RESTO_PRAT);

         SET @V_EXTRA_EMB = @V_EXTRA_EMB + (@V_VRQTDEPRATEMB_ESCOLHIDA - @V_RESTO_PRAT);
         
         SET @V_RESTO_PRAT = 0;

      END;
      
      /*Verifica se a distribuio no  recursiva, inclui uma embalagem incompleta e aloca os restos*/
      IF (@P_TIPO_DIST_AUTO = 'DIST') OR (@V_QTDE_EMB = 0)  BEGIN
         /*INCLUI MAIS UMA EMBALAGEM, INCOMPLETA*/
         SET @V_QTDE_EMB = @V_QTDE_EMB + 1;         
         
         SET @V_QTDE_PRAT = @V_QTDE_PRAT + @V_RESTO_PRAT;
                  
         /*Comensais correspondentes ao resto da diviso*/
         SET @V_C = ROUND(@V_RESTO_PRAT/@V_QTPORPESSOA, 5);
         IF @V_C > @V_TOTAL_COMENSAIS BEGIN
            SET @V_C = @V_TOTAL_COMENSAIS;
         END;
         SET @V_COMENSAIS = @V_COMENSAIS + @V_C;
         
         SET @V_EXTRA_EMB = @V_EXTRA_EMB + @V_RESTO_A;
      
      END;
      
      /*Atualiza o total a distribuir, a quantidade de amostra a distribuir e os comensais correspondentes*/
      SET @V_TOTAL_A_DISTRIBUIR = ROUND(@V_TOTAL_A_DISTRIBUIR - @V_QTDE_PRAT, 5);
      SET @V_TOTAL_COMENSAIS = ROUND(@V_TOTAL_COMENSAIS - @V_COMENSAIS, 5);
      SET @V_QTEXTRA = ROUND(@V_QTEXTRA - @V_EXTRA_EMB, 5);
      
      /*Se no houver mais valores a distribuir, joga o restante dos comensais na embalagem*/
      IF @V_TOTAL_A_DISTRIBUIR = 0 BEGIN
         SET @V_COMENSAIS = @V_COMENSAIS + @V_TOTAL_COMENSAIS;
         SET @V_TOTAL_COMENSAIS = 0;
      END;
      
      
      /*========================================================================*/
      /*=====  INCLUSO DA EMBALAGEM ESCOLHIDA NA DISTRIBUIO DO PRATO ======*/      
      /*=======================================================================*/
            
      
      /*Se prato j est relacionado  embalagem escolhida, atualiza - a com a nova quantidade*/
      IF @V_NREMBDISTCARD_J_EXISTENTE IS NULL BEGIN
      
         /*Busca novo sequencial*/               
         SET @V_CHAVE = 'EMBDISTCARD';
         EXEC NOVO_CODIGO_SC @V_CHAVE, @V_SEQUENCIAL OUTPUT;
         SET @V_NREMBDISTCARD = CONVERT(NUMERIC(20,5), @V_SEQUENCIAL);

         /*Insere embalagem na distribuio do cardpio*/
         INSERT INTO EMBDISTCARD
         (NRDISTCARD, NREMBDISTCARD, CDCONTAINER, QTCONTAINER, IDCADAUTO)
         VALUES
         (@P_NRDISTCARD, @V_NREMBDISTCARD, @V_CDCONTAINER_ESCOLHIDA, @V_QTDE_EMB, 'S');
         
         
         /*Insere o prato na embalagem na distribuio*/
         INSERT INTO ITEMBDISTCARD
         (NRDISTCARD, NRSEQPRATCAR, NREMBDISTCARD, 
          QTCOMENSAIS, QTPRATO, QTEXTRA, IDCADAUTO, IDPRINCIPAL)
         VALUES
         (@P_NRDISTCARD, @P_NRSEQPRATCAR, @V_NREMBDISTCARD, 
          @V_COMENSAIS, @V_QTDE_PRAT, @V_EXTRA_EMB, 'S', 'S');
      END
      ELSE BEGIN

         /*Atualiza a quantidade da embalagem j existente na distribuio*/
         UPDATE EMBDISTCARD
         SET QTCONTAINER = QTCONTAINER + @V_QTDE_EMB
         WHERE NREMBDISTCARD = @V_NREMBDISTCARD_J_EXISTENTE;
         
         /*Atualiza o prato */
         UPDATE ITEMBDISTCARD
            SET QTCOMENSAIS = QTCOMENSAIS + @V_COMENSAIS,
                QTPRATO = QTPRATO + @V_QTDE_PRAT,
                QTEXTRA = QTEXTRA + @V_EXTRA_EMB,
                IDCADAUTO = 'S'
          WHERE NREMBDISTCARD = @V_NREMBDISTCARD_J_EXISTENTE
           AND NRSEQPRATCAR = @P_NRSEQPRATCAR;
      
      END;

   END;
   
END;
GO

-- VERSAO --
EXEC ATUVEROBJ 'DIST_AUTO_PRAT','5.75.000';
GO